from flask_appbuilder.security.manager import AUTH_OID
from flask_appbuilder.security.sqla.manager import SecurityManager
from flask_oidc import OpenIDConnect
from .views import AuthOIDCView
from logging import getLogger
log = getLogger(__name__)


class OIDCSecurityManagerMixin:

    def __init__(self, appbuilder):
        super().__init__(appbuilder)
        if self.auth_type == AUTH_OID:
            self.oid = OpenIDConnect(self.appbuilder.get_app)
            self.authoidview = AuthOIDCView


class OIDCSecurityManager(OIDCSecurityManagerMixin, SecurityManager):
    pass


try:
    from airflow.www_rbac.security import AirflowSecurityManager

    class AirflowOIDCSecurityManager(OIDCSecurityManagerMixin,
                                     AirflowSecurityManager):
        pass
except ImportError:
    log.debug('Airflow not installed')

try:
    from superset.security import SupersetSecurityManager

    class SupersetOIDCSecurityManager(OIDCSecurityManagerMixin,
                                      SupersetSecurityManager):
        pass
except ImportError:
    log.debug('Superset not installed')



import os
from flask import redirect, request
from flask_appbuilder.security.views import AuthOIDView
from flask_login import login_user
from flask_admin import expose
from urllib.parse import quote


# Set the OIDC field that should be used as a username
USERNAME_OIDC_FIELD = os.getenv('USERNAME_OIDC_FIELD', default='sub')
FIRST_NAME_OIDC_FIELD = os.getenv('FIRST_NAME_OIDC_FIELD',
                                  default='nickname')
LAST_NAME_OIDC_FIELD = os.getenv('LAST_NAME_OIDC_FIELD',
                                 default='name')


class AuthOIDCView(AuthOIDView):

    @expose('/login/', methods=['GET', 'POST'])
    def login(self, flag=True):

        sm = self.appbuilder.sm
        oidc = sm.oid

        @self.appbuilder.sm.oid.require_login
        def handle_login():
            user = sm.auth_user_oid(oidc.user_getfield('email'))

            if user is None:
                info = oidc.user_getinfo(
                    [USERNAME_OIDC_FIELD, 'name', 'email', 'nickname']
                )

                user = sm.add_user(
                    username=info.get(USERNAME_OIDC_FIELD),
                    first_name=info.get(FIRST_NAME_OIDC_FIELD),
                    last_name=info.get(LAST_NAME_OIDC_FIELD),
                    email=info.get('email'),
                    role=sm.find_role(sm.auth_user_registration_role)
                )

            login_user(user, remember=False)
            return redirect(self.appbuilder.get_url_for_index)

        return handle_login()

    @expose('/logout/', methods=['GET', 'POST'])
    def logout(self):

        oidc = self.appbuilder.sm.oid

        oidc.logout()
        super(AuthOIDCView, self).logout()
        redirect_url = request.url_root.strip(
            '/') + self.appbuilder.get_url_for_login

        logout_uri = oidc.client_secrets.get(
            'issuer') + '/protocol/openid-connect/logout?redirect_uri='
        if 'OIDC_LOGOUT_URI' in self.appbuilder.app.config:
            logout_uri = self.appbuilder.app.config['OIDC_LOGOUT_URI']

        return redirect(logout_uri + quote(redirect_url))



import os
import sys
from setuptools import setup, find_packages


def fpath(name):
    return os.path.join(os.path.dirname(__file__), name)


def read(fname):
    return open(fpath(fname)).read()


def desc():
    return read('README.md')


setup(
    name='fab_oidc',
    version='0.0.8',
    url='https://github.com/ministryofjustice/fab-oidc/',
    license='MIT',
    author='ministryofjustice',
    author_email='dev@digital.justice.gov.uk',
    description='Flask-AppBuilder SecurityManager for OpenIDConnect',
    long_description=desc(),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    package_data={'': ['LICENSE']},
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=[
        'Flask-AppBuilder>=1.5.0',
        'Flask-OIDC>=1.2.0',
        'Flask-Admin>=1.4.1'
    ],
    tests_require=[
        'nose>=1.0',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 3',
    ],
    test_suite='nose.collector'
)



from werkzeug.contrib.cache import BaseCache
from collections import UserDict


class WerkzeugCacheBackedCredentialStore(UserDict):

    def __init__(self, cache):
        self.cache = cache # type: BaseCache

    def __setitem__(self, key, value):
        return self.cache.set(key, value)

    def __getitem__(self, key):
        return self.cache.get(key)

    def __contains__(self, key):
        return self.cache.has(key)

    def __repr__(self):
        return 'WerkzeugCache: {}'.format(str(self.cache.__class__))
